#include <avr/io.h>
//#include <printf.h>
#include <util/atomic.h>
#include <main.h>
#include <stdbool.h>
#include <avr/sleep.h>
#include <timer.h>
#include <encoder.h>
#include <adc.h>
#include <pwm.h>
#include <uart.h>
#include <message.h>
#include <median_filter.h>

static uint8_t button_state;
static int8_t encoder_count;
static bool updateLuminance = false;
static bool sendLuminance = false;
static bool updateBacklight = false;

BUTTONS_CALLBACK {
	button_state |= button;
}

ENCODER_CALLBACK {
	encoder_count += count;
}

TIMER_16MS_CALLBACK {
	buttonsScan();
	encoderScan();
	updateBacklight = true;
}

TIMER_128MS_CALLBACK {
	adcStart(ADC_3);
	updateLuminance = true;
}

TIMER_1S_CALLBACK {
	sendLuminance = true;
}

int main() {
	uint8_t buffer;
	int8_t encoder;
	bool active;
	bool update_pwm;
	bool exit;
	uint16_t backlight_end = PWM_TOP/2;
	uint16_t backlight = backlight_end;

	uint16_t color_end[3] = {0,0,0};
	uint16_t color[3] = {0,0,0};

	set_sleep_mode(SLEEP_MODE_IDLE);

	//printfInit();
	timerInit();
	buttonsInit();
	encoderInit();
	pwmInit();
	uartInit();

	pwmStart();

	pwmSetDutyCycle(PWM_BL, backlight);
	pwmUpdate();

	sei();

	while (1) {
		active = false;

		// check button presses
		ATOMIC_BLOCK (ATOMIC_FORCEON) {
			buffer = button_state;
			button_state = 0;
		}
		if (buffer != 0) {
			message_t *message;

			message = messageGetSendBuffer();
			message->button.value = buffer;
			messageSend(BUTTON_MSG, sizeof(button_message_t));

			active = true;
		}

		// check encoder
		ATOMIC_BLOCK (ATOMIC_FORCEON) {
			encoder = encoder_count;
			encoder_count = 0;
		}
		if (encoder != 0) {
			message_t *message;

			// print encoder

			message = messageGetSendBuffer();
			message->encoder.value = encoder;
			messageSend(ENCODER_MSG, sizeof(encoder_message_t));

			active = true;
		}

		if (updateBacklight) {
			updateBacklight = false;
			update_pwm = false;

			if (backlight != backlight_end) {
				if (backlight > backlight_end) {
					if ((backlight - backlight_end) > 50) {	// fast shutdown
						backlight -= 5;
					}
					else {
						backlight--;
					}
				}
				else {
					backlight++;
				}

				pwmSetDutyCycle(PWM_BL, backlight);
				update_pwm = true;
			}

			for (uint8_t i=0; i<3; i++) {
				if (color[i] != color_end[i]) {
					if (color[i] < color_end[i]) {
						color[i]++;

						if (color[i] < 5) {
							color[i] = 5;
						}
					}
					else {
						color[i]--;

						if (color[i] < 5) {
							color[i] = 0;
						}
					}

					pwmSetDutyCycle(i+1, color[i]);
					update_pwm = true;
				}
			}

			if (update_pwm) {
				pwmUpdate();
			}

			active = true;
		}

		// filter luminance
		if (updateLuminance) {
			updateLuminance = false;
			filterWrite(adcGet(ADC_3));
			active = true;
		}

		// send luminance
		if (sendLuminance) {
			message_t *message;

			sendLuminance = false;

			message = messageGetSendBuffer();
			message->luminance.value = filterGetValue();
			messageSend(LUMINANCE_MSG, sizeof(luminance_message_t));

			//printf("l: %d\n", luminance);
			active = true;
		}

		exit = false;
		while (uartPending() && !exit) {
			const message_t *message;
			message_type_t type;
			uint8_t c;


			c = uartGet();
			type = messageParse(c, &message);
			switch (type) {
				case INVALID:
					break;
				case BL_MSG:
					backlight_end = message->bl.value;
					exit = true;
					break;
				case RGB_MSG:
					color_end[0] = message->rgb.green;
					color_end[1] = message->rgb.red;
					color_end[2] = message->rgb.blue;

					for (uint8_t i=0; i<3; i++) {
						if (color_end[i] != 0 && color_end[i] < 5) {
							color_end[i] = 5;
						}
					}
					exit = true;
					break;
				default:
					exit = true;
					break;
			}

			active = true;
		}

		// its not really clean to sleep without further conditions
		// but this solution is sufficient here
		if (active == false) {
        	sleep_mode();
        }
	}
}

