#include "timer.h"

#include <avr/interrupt.h>
#include <util/atomic.h>

static uint32_t time = 0;

void timerInit() {	
	// init timer
	TCCR0A = (1<<WGM01);
	TCCR0B = (1<<CS02) | (1<<CS00);	// CTC, 1024 prescaler
	OCR0A = 12;						// ~1ms
	TIMSK0 |= (1<<OCIE0A);
}

uint32_t timerTime(void) {
	uint32_t buffer;
	
	ATOMIC_BLOCK (ATOMIC_RESTORESTATE) {
		buffer = time;
	}

	return buffer;
}

uint32_t timerTimeISR(void) {
	return time;
}

ISR(TIMER0_COMPA_vect ) {
	time++;

	if ((time & 0xF) == 0) {		// ~16 ms tick
		TIMER_16MS();
	}

	if ((time & 0x7F) == 0) {		// ~128 ms tick
		TIMER_128MS();
	}

	if ((time & 0x3FF) == 0) {		// ~1 s tick
		TIMER_1S();
	}
	
	//tick();
}
