#include <buttons.h>

#define PORT	PORTB
#define PIN		PINB
#define DDR		DDRB

#define PORT_MASK	((1<<BT1)|(1<<BT2)|(1<<BT3)|(1<<BT4))

void buttonsInit() {
	DDR &= ~PORT_MASK;
	PORT |= PORT_MASK;
}

void buttonsScan() {
	static uint8_t ct0 = 0xff, ct1 = 0xff;
	static uint8_t button_state = 0;
	static uint8_t last_button_state = 0;
	uint8_t i;
	uint8_t buttons;

	// debounce
 	// this function must be called periodically and
 	// returns the debounced state of one port
 	// (it does not detect events, like button pressed/released)
	i = button_state ^ (~PIN & PORT_MASK);			// button changed ?
	ct0 = ~( ct0 & i );            					// reset or count ct0
	ct1 = ct0 ^ (ct1 & i);         					// reset or count ct1
	i &= ct0 & ct1;                					// count until roll over ?
	button_state ^= i;  	       					// then toggle debounced state

	// detect button presses
	buttons = button_state ^ last_button_state;		// detect edge
	buttons &= button_state;						// select rising edges
	last_button_state = button_state;

	if (buttons != 0) {
		buttonsCallback(buttons);
	}
}