#include <encoder.h>
#include <avr/interrupt.h>
#include <util/atomic.h>
#include <timer.h>

#define PORT	PORTD
#define PIN		PIND
#define DDR		DDRD

#define PORT_MASK	((1<<PD2)|(1<<PD3))

static uint8_t isr_count = 0;
static direction_t isr_dir = UP;
static uint8_t edge = 0;

void encoderInit() {
	DDR &= ~PORT_MASK;
	PORT |= PORT_MASK;

	EICRA = (0<<ISC11) | (1<<ISC10) | (0<<ISC01) | (1<<ISC00);
	EIMSK |= (1<<INT1) | (1<<INT0);
}

void encoderScan() {
	int16_t count;
	uint8_t dir;

	ATOMIC_BLOCK (ATOMIC_RESTORESTATE) {
		count = isr_count;
		isr_count = 0;
		dir = isr_dir;
	}

	if (count != 0) {
		if (count > 127) {
			count = 127;
		}
		count *= dir;
		encoderCallback((int8_t)count);
	}
}

ISR(INT0_vect) {
	edge = 1;
}

ISR(INT1_vect) {
	static uint32_t last_edge = 0;
	uint32_t time;
	uint8_t count;
	uint32_t time_diff;
	uint8_t pin;

	pin = PIN;
	time = timerTime();

	sei();									// do not disturb pwm module

	if ((pin & (1<<PD3)) != 0) {			// rising edge
		if (edge == 1) {
			time_diff = time - last_edge;

			if (time_diff > 20) {
				count = 1;
			}
			else if (time_diff < 12) {
				count = 10;
			}
			else {
				count = 3;
				//count = -10L * (int8_t)(time_diff-10L) / 140L + 10;
			}

			isr_count += count;
			last_edge = time;

			if ((pin & (1<<PD2)) != 0) {
				isr_dir = UP;
			}
			else {
				isr_dir = DOWN;
			}
		}
	}

	edge = 0;
}

