#
# avr-gcc Makefile
#
# author:	Andreas Hagmann <ahagmann@ecs.tuwien.ac.at>
# date:		20.01.2012
#

MCU ?= atmega88
INSTALL ?= install_flash
F_CPU ?= 12000000UL

DEFS ?= F_CPU=$(F_CPU)
CC_FLAGS ?= -Os -Wall -funsigned-char -funsigned-bitfields -fpack-struct -fshort-enums -fno-tree-scev-cprop -ffunction-sections -fdata-sections -frename-registers
LD_FLAGS ?= -Wl,--relax -Wl,--gc-sections -Wl,-u,vfprintf -lprintf_min

DEFS += $(ADD_DEFS)
CC_FLAGS += $(ADD_CC_FLAGS) $(addprefix -D, $(DEFS)) -std=gnu99
LD_FLAGS += $(ADD_LD_FLAGS)

# bootloader has wrong signature, thus atmega8 is used here:x
PROG_FLAGS += -c avr109 -P /dev/ttyAMA0 -b 57600 -p atmega8

BUILD_TYPE ?= default

SRC_DIR = ./src
BIN_DIR = ./bin
LIST_DIR = ./list
BUILD_DIR = ./build
LIB_BUILD = ./a

ROOT_MAKEFILE := $(firstword $(MAKEFILE_LIST))
PATH_TO_ME := $(dir $(lastword $(MAKEFILE_LIST)))

LIB_DIR ?= $(PATH_TO_ME)../lib

LIB_DIRS := $(addprefix $(LIB_DIR)/, $(LIBS))
CFILES := $(shell ls $(SRC_DIR)/*.c 2> /dev/null)
CFILES += $(foreach lib, $(LIB_DIRS), $(shell ls $(lib)/*.c 2> /dev/null))

VPATH := $(SRC_DIR) $(LIB_DIRS)
OUT := $(BIN_DIR)/$(PROJ_NAME).elf
CFILE_NAMES := $(basename $(notdir $(CFILES)))
OBJS := $(addprefix $(BUILD_DIR)/, $(addsuffix .o, $(CFILE_NAMES)))
DEPS := $(shell ls $(BUILD_DIR)/*.d 2> /dev/null)
OUT_HEX :=  $(BIN_DIR)/$(PROJ_NAME).ihex

.SUFFIXES:

.PHONY: all size clean distclean whole hex

all: $(OUT)

hex: $(OUT_HEX)

-include $(DEPS)
-include $(PATH_TO_ME)$(BUILD_TYPE).mk

$(BIN_DIR):
	@echo create directory: $@
	@mkdir $(BIN_DIR)

$(BUILD_DIR):
	@echo create directory: $@
	@mkdir $(BUILD_DIR)

$(LIST_DIR):
	@echo create directory: $@
	@mkdir $(LIST_DIR)

$(OUT_HEX): $(OUT)
	@avr-objcopy -j .text -j .data -O ihex $< $@ 

install_flash: $(OUT_HEX)
	@flash $(PROG_FLAGS) -U flash:w:$<

#install_eeprom: $(OUT_HEX)
#	@flash $(PROG_FLAGS) -U eeprom:w:$<

install: $(INSTALL)

size: $(OUT)
#	@avr-size --mcu=$(MCU) --format=avr $(OUT)
	@avr-size -A $(OUT)

clean:
	rm -f $(BUILD_DIR)/*.o $(BUILD_DIR)/*.d
	rm -rf $(BUILD_DIR)
	rm -f $(LIST_DIR)/*.list
	rm -rf $(LIST_DIR)
	rm -rf $(LIB_BUILD)

distclean: clean
	rm -f $(EEPROM)
	rm -f $(FLASH)
	rm -f $(OUT)
	rm -rf $(BIN_DIR)
