#include <message.h>
#include <uart.h>

static uint8_t msgRcvBuffer[MSG_BUFFER_SIZE];
static uint8_t msgSndBuffer[MSG_BUFFER_SIZE+2];

static uint8_t calcChecksum(uint8_t *buffer, uint8_t len);

message_type_t messageParse(uint8_t c, const message_t **buffer) {
    static uint8_t pointer = 0;
    static uint8_t len = 0;
    static message_type_t type;
    static parse_state_t state = WAIT_START;
    uint8_t cs;

    //if (c == START_CHAR) {       // new message arives
    //    len = 0;
    //    state = WAIT_LEN;
    //}
    //else {
        switch (state) {
        case WAIT_START:
            if (c == START_CHAR) {
                state = WAIT_LEN;
            }
            pointer = 0;
            break;

        case WAIT_LEN:
            if ((c < MAX_MSG_LEN) && (c > 0)){
                len = c;
                msgRcvBuffer[pointer] = c;
                pointer++;
                state = WAIT_TYPE;
            }
            else {
                state = WAIT_START;
            }
            break;

        case WAIT_TYPE:
            state = READ;
            type = (message_type_t)c;
            msgRcvBuffer[pointer] = c;
            pointer++;
            break;

        case READ:
            msgRcvBuffer[pointer] = c;
            pointer++;
            if (pointer == len+2) {
                state = WAIT_CHECKSUM;
            }
            break;

        case WAIT_CHECKSUM:
            cs = calcChecksum(msgRcvBuffer, len+2);
            state = WAIT_START;
            if (c == cs) {
                *buffer = (message_t*)(msgRcvBuffer+2);
                //msgRcvBuffer[len+2] = 0;    // 0 terminated
                return type;
            }
            break;
        }
    //}

    return INVALID;
}

uint8_t calcChecksum(uint8_t *buffer, uint8_t len) {
    uint8_t cs = 8;
    uint8_t i;

    for (i=0; i<len; i++) {
        cs = cs + buffer[i];
    }

    return cs;
}

message_t *messageGetSendBuffer() {
    return (message_t*)(msgSndBuffer+3);      // skip start byte, length and type
}

// prepare the send buffer (add start byte and checksum)
void messageSend(message_type_t type, uint8_t len) {
    msgSndBuffer[0] = START_CHAR;
    msgSndBuffer[1] = len;
    msgSndBuffer[2] = type;
    msgSndBuffer[len+3] = calcChecksum(msgSndBuffer+1, len+2);
    
    for (uint8_t i=0; i<len+4; i++) {
        uartSend(msgSndBuffer[i]);
    }
}
