
#include "uart.h"
#include <util/atomic.h>
#include <avr/interrupt.h>

#include <util/setbaud.h>

#define BUFFER_SIZE 16

static uint8_t buffer[BUFFER_SIZE];
static uint8_t read_p = 0;
static uint8_t write_p = 0;

void uartInit() {
    UBRR0L = UBRRL_VALUE;
    UBRR0H = UBRRH_VALUE;

	UCSR0A = (USE_2X<<U2X0);
	UCSR0C = (1<<UCSZ01) | (1<<UCSZ00);	// no parity, 8 data bits
	UCSR0B = (1<<TXEN0) | (1<<RXEN0) | (1<<RXCIE0);

	read_p = 0;
	write_p = 0;
}

void uartSend(uint8_t c) {
	while ((UCSR0A & (1<<UDRE0)) == 0);
	UDR0 = c;
}

bool uartPending(void) {
	bool pending;

	ATOMIC_BLOCK (ATOMIC_FORCEON) {
		pending = (read_p == write_p);
	}
	return !pending;
}

uint8_t uartGet(void) {
	uint8_t c;

	ATOMIC_BLOCK (ATOMIC_FORCEON) {
		c = buffer[read_p];
		read_p = (read_p + 1) % BUFFER_SIZE;
	}
	return c;
}

ISR(USART_RX_vect) {
	uint8_t write_p_new;

	write_p_new = (write_p + 1) % BUFFER_SIZE;
	if (write_p_new != read_p) {	// add to buffer
		buffer[write_p] = UDR0;
		write_p = write_p_new;
	}
	else {							// drop value
		uint8_t dummy = UDR0;
		(void)dummy;
	}
}