from PyQt4 import QtCore
from datetime import datetime

class Alarm:
	def __init__(self, key):
		self._time = datetime.today().time()
		self._date = None
		self._key = key
		self._active = False
		self._timeout = 0

	def set_time(self, time, date=None):
		self._time = time
		self._date = date

	def time(self):
		return self._time

	def date(self):
		return self._date

	def enable(self):
		self._active = True

	def disable(self):
		self._active = False

	def active(self):
		return self._active

	def decrement_timeout(self):
		if self._timeout > 0:
			self._timeout = self._timeout

	def reload_timeout(self):
		self._timeout = 4

	def timeout_ok(self):
		return (self._timeout == 0)


class AlarmControl(QtCore.QObject):
	minTick = QtCore.pyqtSignal(datetime)
	alarm = QtCore.pyqtSignal()

	def __init__(self, parent=None):
		super().__init__(parent)
		self.alarms = {}

		self.lastMinute = 0

		self.tickTimer = QtCore.QTimer()
		self.tickTimer.timeout.connect(self._secTick)
		self.tickTimer.setInterval(1000)
		self.tickTimer.start()

		self.minTick.connect(self._clock_tick_1m)

	def _secTick(self):
		now = datetime.today()
		
		if self.lastMinute != now.minute:
			self.minTick.emit(now)

		self.lastMinute = now.minute

	def setTime(self, alarm, time, date=None):
		self.alarms[alarm].set_time(time, date)

	def time(self, alarm):
		return self.alarms[alarm].time()

	def enable(self, alarm):
		self.alarms[alarm].enable()

	def disable(self, alarm):
		self.alarms[alarm].disable()

	def delete(self, alarm):
		del self.alarms[key]

	def add(self, alarm):
		self.alarms[alarm] = Alarm(alarm)

	def enabled(self, alarm):
		return self.alarms[alarm].active()

	def _clock_tick_1m(self, date_time):
		now = date_time.time()
		#print("now: ", now)
		
		for a in self.alarms.values():
			#a.decrement_timeout()

			if a.active():

				#if a.date():
				#	alarm_time = a.time()
				#	print("alarm time with date: ", alarm_time)
				#else:
				#	alarm_time = a.time()
				#	print("alarm time without date: ", alarm_time)

				if now.minute == a.time().minute and now.hour == a.time().hour:
					#if a.timeout_ok():
						# emit wake signal to start wake procedure
						#a.reload_timeout()
					self.alarm.emit()

					# if date is specified delete alarm
					if a.date():
						del self.alarms[a.key]
