from PyQt4 import QtCore
from datetime import time
from datetime import datetime
from sound import Sound
from settings import Settings
import subprocess

class State:
	IDLE = 0
	EDIT_1 = 1
	EDIT_2 = 2
	ALARM = 3

class GuiControl(QtCore.QObject):
	showMessage = QtCore.pyqtSignal(str, int)
	clearMessage = QtCore.pyqtSignal()
	toggleMenu = QtCore.pyqtSignal()
	setAlarmTime1 = QtCore.pyqtSignal(str)
	setAlarmTime2 = QtCore.pyqtSignal(str)
	setAlarmStatus1 = QtCore.pyqtSignal(bool)
	setAlarmStatus2 = QtCore.pyqtSignal(bool)
	setBlinkStatus1 = QtCore.pyqtSignal(bool)
	setBlinkStatus2 = QtCore.pyqtSignal(bool)
	setDayColors = QtCore.pyqtSignal()
	setNightColors = QtCore.pyqtSignal()
	setBrightness = QtCore.pyqtSignal(int)
	updateWeather = QtCore.pyqtSignal()
	setLight = QtCore.pyqtSignal(int, int, int)

	def __init__(self, alarmControl, parent=None):
		super().__init__(parent)

		self.alarms = alarmControl
		self.alarms.add('a1')
		self.alarms.add('a2')

		self.state = State.IDLE
		self.tempTime = 0

		self.TIME_FORMAT = '{:d}:{:02d}'

		self.sound = Sound()
		self.settings = Settings()

		self.alarms.setTime('a1', time(int(self.settings.alarm1_hours), self.settings.alarm1_minutes))
		self.alarms.setTime('a2', time(int(self.settings.alarm2_hours), self.settings.alarm2_minutes))
		if self.settings.alarm1_active:
			self._enable1()
		if self.settings.alarm2_active:
			self._enable2()

		self.alarms.alarm.connect(self.alarm)

		self.lastLuminance = 0

		self.light = False

	def _startEdit1(self):
		time = self.alarms.time('a1')
		self.tempTime = time.minute + time.hour*60
		self.setBlinkStatus1.emit(True)
		self.state = State.EDIT_1

	def _startEdit2(self):
		time = self.alarms.time('a2')
		self.tempTime = time.minute + time.hour*60
		self.setBlinkStatus2.emit(True)
		self.state = State.EDIT_2

	def _showAlarmMessage(self, time):
		now = datetime.today().time()
		diff = ((time.minute+time.hour*60) - (now.minute+now.hour*60)) % (24*60)
		if diff < 60:
			msg = "Wecken in {:d} Minuten".format(diff)
		else:
			msg = "Wecken in {:d}:{:02d} Stunden".format(int(diff/60), diff%60)
		self.showMessage.emit(msg, 5000)

	def _showShortAlarmMessage(self, time):
		now = datetime.today().time()
		diff = ((time.minute+time.hour*60) - (now.minute+now.hour*60)) % (24*60)
		if diff < 60:
			msg = "noch {:d} Minuten".format(diff)
		else:
			msg = "noch {:d}:{:02d} Stunden".format(int(diff/60), diff%60)
		self.showMessage.emit(msg, 10000)

	def _stopEdit1(self):
		self.alarms.setTime('a1', time(int(self.tempTime/60), self.tempTime%60))
		self.settings.alarm1_minutes = self.tempTime%60
		self.settings.alarm1_hours = int(self.tempTime/60)
		self.settings.store()
		self.setBlinkStatus1.emit(False)
		self._enable1()
		self.state = State.IDLE
		self.refresh()
		self._showAlarmMessage(time(int(self.tempTime/60), self.tempTime%60))

	def _stopEdit2(self):
		self.alarms.setTime('a2', time(int(self.tempTime/60), self.tempTime%60))
		self.settings.alarm2_minutes = self.tempTime%60
		self.settings.alarm2_hours = int(self.tempTime/60)
		self.settings.store()
		self.setBlinkStatus2.emit(False)
		self._enable2()
		self.state = State.IDLE
		self.refresh()
		self._showAlarmMessage(time(int(self.tempTime/60), self.tempTime%60))

	def _cancel1(self):
		self.setBlinkStatus1.emit(False)
		self.state = State.IDLE
		self.clearMessage.emit()
		self.refresh()

	def _cancel2(self):
		self.setBlinkStatus2.emit(False)
		self.state = State.IDLE
		self.clearMessage.emit()
		self.refresh()

	def _enable1(self):
		self.setAlarmStatus1.emit(True)
		self.settings.alarm1_active = True
		self.settings.store()
		self.alarms.enable('a1')

	def _enable2(self):
		self.setAlarmStatus2.emit(True)
		self.settings.alarm2_active = True
		self.settings.store()
		self.alarms.enable('a2')

	def _disable1(self):
		self.setAlarmStatus1.emit(False)
		self.settings.alarm1_active = False
		self.settings.store()
		self.alarms.disable('a1')

	def _disable2(self):
		self.setAlarmStatus2.emit(False)
		self.settings.alarm2_active = False
		self.settings.store()
		self.alarms.disable('a2')

	def _toggle1(self):
		if self.alarms.enabled('a1'):
			self._disable1()
		else:
			self._enable1()

	def refresh(self):
		self.setAlarmStatus1.emit(self.alarms.enabled('a1'))
		self.setAlarmStatus2.emit(self.alarms.enabled('a2'))
		time = self.alarms.time('a1')
		self.setAlarmTime1.emit(self.TIME_FORMAT.format(time.hour, time.minute))
		time = self.alarms.time('a2')
		self.setAlarmTime2.emit(self.TIME_FORMAT.format(time.hour, time.minute))

	def alarm1Click(self):
		if self.state == State.IDLE:
			self._startEdit1()
		elif self.state == State.EDIT_1:
			self._stopEdit1()

	def alarm2Click(self):
		if self.state == State.IDLE:
			self._startEdit2()
		elif self.state == State.EDIT_2:
			self._stopEdit2()

	def active1Click(self):
		if (self.state == State.IDLE):
			self._toggle1()

	def active2Click(self):
		if self.state == State.IDLE:
			if self.alarms.enabled('a2'):
				self._disable2()
			else:
				self._enable2()

	def timeClick(self):
		self.toggleLight()

	def wlanOnClick(self):
		subprocess.call(['ifup', 'wlan0'])

	def wlanOffClick(self):
		subprocess.call(['ifdown', 'wlan0'])

	def button1(self):
		if self.state == State.IDLE:
			self._startEdit1()
		elif self.state == State.EDIT_1:
			self._cancel1()
		elif self.state == State.EDIT_2:
			self._cancel2()

	def button2(self):
		if self.state == State.IDLE:
			self.toggleMenu.emit()

	def buttonE(self):
		if self.state == State.IDLE:
			self.toggleLight()
		elif self.state == State.EDIT_1:
			self._stopEdit1()
		elif self.state == State.EDIT_2:
			self._stopEdit2()
		elif self.state == State.ALARM:
			self.sound.stopDecreasing()
			self.state = State.IDLE

	def encoder(self, value):
		self.tempTime = (self.tempTime - value) % (24*60)
		time_str = self.TIME_FORMAT.format(int(self.tempTime/60), self.tempTime%60)

		if self.state == State.EDIT_1:
			self.setAlarmTime1.emit(time_str)
			self._showShortAlarmMessage(time(int(self.tempTime/60), self.tempTime%60))
		elif self.state == State.EDIT_2:
			self.setAlarmTime2.emit(time_str)
			self._showShortAlarmMessage(time(int(self.tempTime/60), self.tempTime%60))

	def alarm(self):
		self._cancel1()
		self._cancel2()

		print("alarm")

		if self.state == State.IDLE:
			self.state = State.ALARM
			self.sound.playIncreasing(self.settings.sound)

	def toggleLight(self):
		self.light = not self.light
		self._updateLight()

	def lightOn(self):
		self.light = True
		self._updateLight()

	def lightOff(self):
		self.light = False
		self._updateLight()

	def _updateLight(self):
		self.settings.store()
		if self.light:
			self.setLight.emit(self.settings.red, self.settings.green, self.settings.blue)
		else:
			self.setLight.emit(0, 0, 0)

	def setRed(self, value):
		self.settings.red = value
		self._updateLight()

	def setGreen(self, value):
		self.settings.green = value
		self._updateLight()

	def setBlue(self, value):
		self.settings.blue = value
		self._updateLight()

	def updateLuminance(self, value):
		b = -9.505*value + 2235.6

		b = min(b, 1023)
		b = max(b, 2)
		b = int(b)

		#print("brightness", str(value), '->', str(b))
		#self.setBrightness.emit(b)

		if value < 230:
			self.setBrightness.emit(1023)
		else:
			self.setBrightness.emit(b)

		if self.lastLuminance >= 230 and value < 230:
			self.setDayColors.emit()
		elif self.lastLuminance <= 230 and value > 230:
			self.setNightColors.emit()

		self.lastLuminance = value

	def timeAction(self, date_time):
		if date_time.hour <= 22 and date_time.hour >= 8 and date_time.minute == 5:
			self.updateWeather.emit()

		if date_time.hour == 22 and date_time.minute == 10:
			self.wlanOffClick()

		if date_time.hour == 8 and date_time.minute == 2:
			self.wlanOnClick()
