from PyQt4 import QtCore, QtGui
from qt_main import *
from datetime import datetime
from settings import Settings
from menu_widget import MenuWidget
import os
import time

class MainWindow(QtGui.QMainWindow):
	updateClock = QtCore.pyqtSignal(datetime)
	showMessage = QtCore.pyqtSignal(str, int)
	clearMessage = QtCore.pyqtSignal()
	alarm1Click = QtCore.pyqtSignal()
	alarm2Click = QtCore.pyqtSignal()
	active1Click = QtCore.pyqtSignal()
	active2Click = QtCore.pyqtSignal()
	timeClick = QtCore.pyqtSignal()
	setAlarmTime1 = QtCore.pyqtSignal(str)
	setAlarmTime2 = QtCore.pyqtSignal(str)
	setAlarmStatus1 = QtCore.pyqtSignal(bool)
	setAlarmStatus2 = QtCore.pyqtSignal(bool)
	setBlinkStatus1 = QtCore.pyqtSignal(bool)
	setBlinkStatus2 = QtCore.pyqtSignal(bool)
	setTemp = QtCore.pyqtSignal(str)
	setFcTemp = QtCore.pyqtSignal(str)
	setFcRain = QtCore.pyqtSignal(str)
	setFcIcon = QtCore.pyqtSignal(str)
	wlanOnClick = QtCore.pyqtSignal()
	wlanOffClick = QtCore.pyqtSignal()
	#lightOnClick = QtCore.pyqtSignal()
	#lightOffClick = QtCore.pyqtSignal()
	redChanged = QtCore.pyqtSignal(int)
	greenChanged = QtCore.pyqtSignal(int)
	blueChanged = QtCore.pyqtSignal(int)

	def __init__(self, parent=None):
		QtGui.QWidget.__init__(self, parent)

		self.settings = Settings()

		self.setFixedSize(320, 240)
		self.stack = QtGui.QStackedWidget()
		self.setCentralWidget(self.stack)

		# first page, main gui
		self.main = MainWidget()
		self.stack.addWidget(self.main)

		# second page, menu
		self.menu = MenuWidget()
		self.stack.addWidget(self.menu)

		# connect
		self.updateClock.connect(self.main.setClock)
		self.showMessage.connect(self.main.showMessage)
		self.clearMessage.connect(self.main.clearMessage)
		self.main.alarm1Click.connect(self.alarm1Click)
		self.main.alarm2Click.connect(self.alarm2Click)
		self.main.timeClick.connect(self.timeClick)
		self.main.active1Click.connect(self.active1Click)
		self.main.active2Click.connect(self.active2Click)
		self.setAlarmTime1.connect(self.main.setAlarmTime1)
		self.setAlarmTime2.connect(self.main.setAlarmTime2)
		self.setAlarmStatus1.connect(self.main.setAlarmStatus1)
		self.setAlarmStatus2.connect(self.main.setAlarmStatus2)
		self.setBlinkStatus1.connect(self.main.setBlinkStatus1)
		self.setBlinkStatus2.connect(self.main.setBlinkStatus2)
		self.setTemp.connect(self.main.setTemp)
		self.setFcTemp.connect(self.main.setFcTemp)
		self.setFcRain.connect(self.main.setFcRain)
		self.setFcIcon.connect(self.main.setFcIcon)

		self.menu.wlanOnClick.connect(self.wlanOnClick)
		self.menu.wlanOffClick.connect(self.wlanOffClick)
		#self.menu.lightOnClick.connect(self.lightOnClick)
		#self.menu.lightOffClick.connect(self.lightOffClick)
		self.menu.redChanged.connect(self.redChanged)
		self.menu.greenChanged.connect(self.greenChanged)
		self.menu.blueChanged.connect(self.blueChanged)

	def _setColors(self, forground, background):
		palette = QtGui.QPalette()
		palette.setColor(QtGui.QPalette.Background, background)
		palette.setColor(QtGui.QPalette.WindowText, forground)
		palette.setColor(QtGui.QPalette.ButtonText, forground)
		self.setPalette(palette)

	def setNightColors(self):
		self._setColors(self.settings.forground_night, self.settings.background_night)
		self.main.setNightColors()

	def setDayColors(self):
		self._setColors(self.settings.forground_day, self.settings.background_day)
		self.main.setDayColors()

	def toggleMenu(self):
		if self.stack.currentIndex() == 0:
			self.stack.setCurrentIndex(1)
		else:
			self.stack.setCurrentIndex(0)


class MainWidget(QtGui.QWidget):
	alarm1Click = QtCore.pyqtSignal()
	alarm2Click = QtCore.pyqtSignal()
	active1Click = QtCore.pyqtSignal()
	active2Click = QtCore.pyqtSignal()
	timeClick = QtCore.pyqtSignal()

	def __init__(self, parent=None):
		QtGui.QWidget.__init__(self, parent)
		self.ui = Ui_main()
		self.ui.setupUi(self)

		self.settings = Settings()

		self.alarm_status1 = False
		self.alarm_status2 = False

		self.night = False

		# message timer
		self.messageTimer = QtCore.QTimer()
		self.messageTimer.timeout.connect(self._cleanupMessage)
		self.messageTimer.setSingleShot(True)

		# blink timer
		self.blinkTimer1 = QtCore.QTimer()
		self.blinkTimer1.timeout.connect(self._blink1)
		self.blinkTimer1.setInterval(300)
		self.blinkTimer2 = QtCore.QTimer()
		self.blinkTimer2.timeout.connect(self._blink2)
		self.blinkTimer2.setInterval(300)

		self.ui.time1.pressed.connect(self.alarm1Click)
		self.ui.time2.pressed.connect(self.alarm2Click)
		self.ui.alarm1.pressed.connect(self.active1Click)
		self.ui.alarm2.pressed.connect(self.active2Click)
		self.ui.time.pressed.connect(self.timeClick)

		# prepare icons
		self.alarm_active_day = self.colorIcon(self.settings.active_alarm_icon, self.settings.forground_day)
		self.alarm_inactive_day = self.colorIcon(self.settings.inactive_alarm_icon, self.settings.forground_day)
		self.alarm_active_night = self.colorIcon(self.settings.active_alarm_icon, self.settings.forground_night)
		self.alarm_inactive_night = self.colorIcon(self.settings.inactive_alarm_icon, self.settings.forground_night)

	def colorIcon(self, icon_path, color):
		base_icon = QtGui.QPixmap(icon_path)
		icon = QtGui.QPixmap(base_icon.size())
		icon.fill(color)
		icon.setMask(base_icon.createMaskFromColor(QtCore.Qt.transparent))
		return QtGui.QIcon(icon)

	def showMessage(self, str, duration=3000):
		self.ui.label.setText(str)
		self.ui.info.setCurrentWidget(self.ui.message)
		self.messageTimer.start(duration)

	def clearMessage(self):
		self.messageTimer.stop()
		self._cleanupMessage()

	def _cleanupMessage(self):
		self.ui.info.setCurrentWidget(self.ui.panel)

	def setClock(self, date_time):
		#print("set clock called", str(date_time))
		self.ui.time.setText(str(date_time.hour) + ':' + str(date_time.minute).zfill(2))

	def setTemp(self, value):
		self.ui.temp.setText(str(value))

	def setFcTemp(self, value):
		self.ui.fctemp.setText(str(value))

	def setFcRain(self, value):
		self.ui.fcrain.setText(str(value))

	def setFcIcon(self, icon):
		if icon != '':
			icon = QtGui.QPixmap(os.path.join(self.settings.weather_icon_path, icon))
			self.ui.fcicon.setPixmap(icon)
		else:
			self.ui.fcicon.clear()

	def setAlarmTime1(self, value):
		self.ui.time1.setText(value)

	def setAlarmTime2(self, value):
		self.ui.time2.setText(value)

	def _updateIcons(self):
		if self.night:
			active = self.alarm_active_night
			inactive = self.alarm_inactive_night
		else:
			active = self.alarm_active_day
			inactive = self.alarm_inactive_day

		if self.alarm_status1:
			icon = active
		else:
			icon = inactive
		self.ui.alarm1.setIcon(icon)

		if self.alarm_status2:
			icon = active
		else:
			icon = inactive
		self.ui.alarm2.setIcon(icon)

	def setAlarmStatus1(self, enable):
		self.alarm_status1 = enable
		self._updateIcons()

	def setAlarmStatus2(self, enable):
		self.alarm_status2 = enable
		self._updateIcons()

	def setBlinkStatus1(self, enable):
		if enable:
			self.blinkTimer1.start()
		else:
			self.blinkTimer1.stop()
			self.ui.time1.setVisible(True)

	def setBlinkStatus2(self, enable):
		if enable:
			self.blinkTimer2.start()
		else:
			self.blinkTimer2.stop()
			self.ui.time2.setVisible(True)

	def _blink1(self):
		self.ui.time1.setVisible(not self.ui.time1.isVisible())

	def _blink2(self):
		self.ui.time2.setVisible(not self.ui.time2.isVisible())

	def _setColors(self, forground, background):
		palette = QtGui.QPalette()
		palette.setColor(QtGui.QPalette.Background, background)
		palette.setColor(QtGui.QPalette.WindowText, forground)
		palette.setColor(QtGui.QPalette.ButtonText, forground)
		self.ui.time1.setPalette(palette)
		self.ui.time2.setPalette(palette)
		self.ui.time.setPalette(palette)
		self._updateIcons()

	def setNightColors(self):
		self.night = True
		self._setColors(self.settings.forground_night, self.settings.background_night)

	def setDayColors(self):
		self.night = False
		self._setColors(self.settings.forground_day, self.settings.background_day)

