
import serial
from PyQt4 import QtCore
import sys

class State():
	WAIT_START = 1
	WAIT_LEN = 2
	WAIT_TYPE = 3
	READ = 4
	WAIT_CHECKSUM = 5

class Type():
	LUMINANCE_MSG = 1
	BUTTON_MSG = 2
	ENCODER_MSG = 3
	BL_MSG = 4
	RGB_MSG = 5

class Message(QtCore.QThread):
	START_CHAR = 0xff
	MSG_BUFFER_SIZE = 40

	button1 = QtCore.pyqtSignal()
	button2 = QtCore.pyqtSignal()
	buttonE = QtCore.pyqtSignal()
	encoder = QtCore.pyqtSignal(int)
	luminance = QtCore.pyqtSignal(int)

	def __init__(self, baudrate=115200, parent=None):
		super().__init__(parent)

		self.state = State.WAIT_START
		self.pointer = 0
		self.len = 0
		self.type = 0
		self.buffer = bytearray(Message.MSG_BUFFER_SIZE) #[None] * Message.MSG_BUFFER_SIZE

		# open serial port
		self.uart = serial.Serial('/dev/ttyAMA0')
		print ("connected to: " + self.uart.portstr)
		self.uart.baudrate = baudrate

	def calcChecksum(self, buffer, len):
		cs = 8;

		for i in range(len):
			cs = (cs + int(buffer[i])) % 0x100
		return cs;

	def parse(self, c):
		#print("parse", hex(c), "in", self.state)
		if self.state == State.WAIT_START:
			if c == Message.START_CHAR:
				self.state = State.WAIT_LEN
			self.pointer = 0;

		elif self.state == State.WAIT_LEN:
			if (c < Message.MSG_BUFFER_SIZE-2) and (c > 0):
				self.len = c
				self.buffer[self.pointer] = c
				self.pointer = self.pointer + 1
				self.state = State.WAIT_TYPE
			else:
				self.state = State.WAIT_START

		elif self.state == State.WAIT_TYPE:
			self.state = State.READ
			self.type = c
			self.buffer[self.pointer] = c
			self.pointer = self.pointer + 1

		elif self.state == State.READ:
			self.buffer[self.pointer] = c
			self.pointer = self.pointer + 1
			if self.pointer == self.len+2:
				self.state = State.WAIT_CHECKSUM

		elif self.state == State.WAIT_CHECKSUM:
			self.state = State.WAIT_START;
			cs = self.calcChecksum(self.buffer, self.len+2)
			if c == cs:
				type = self.buffer[1]
				value = int(self.buffer[2])

				if type == Type.LUMINANCE_MSG:
					#print('luminance', str(value))
					self.luminance.emit(value)

				elif type == Type.BUTTON_MSG:
					#print('pressed', str(value))
					if value & 0x08:
						self.button1.emit()
					if value & 0x04:
						self.button2.emit()
					if value & 0x10:
						self.buttonE.emit()

				elif type == Type.ENCODER_MSG:
					# convert to signed
					if value > 127:
						value = value - 256
					#print('encoder', str(value))
					self.encoder.emit(value)

	def close(self):
		self.uart.close()

	def run(self):
		# wait for new data
		while True:
			line = self.uart.read(1)
			c = int.from_bytes(line, byteorder='big')
			self.parse(c)

	def read(self):
		while(1):
			line = self.uart.read(1)
			i = int.from_bytes(line, byteorder='big')
			print(hex(i), end=' ')
			sys.stdout.flush()

	# convert 16 bit value to 2 bytes
	def toBytes(self, value):
		return [value & 0xff, value>>8 & 0xff]

	def sendMessage(self, type, data):
		buffer = bytearray()

		buffer.append(Message.START_CHAR)
		buffer.append(len(data))
		buffer.append(type)
		data = bytearray(data)
		buffer = buffer + data
		buffer.append(self.calcChecksum(buffer[1:], len(data)+2))

		#print(buffer)
		self.uart.write(buffer)

	def sendRGB(self, red, green, blue):
		self.sendMessage(Type.RGB_MSG, self.toBytes(red) + self.toBytes(green) + self.toBytes(blue))

	def sendBacklite(self, value):
		self.sendMessage(Type.BL_MSG, self.toBytes(value))
